<?php
namespace Controller;

use Util\Utilities as Environment;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;

class Dashboard {
    
    /**
     * @var \Service\Dashboard $dashboardService
     */
    private $dashboardService;

    function __construct(\Service\Dashboard $dashboardService)
    {
        $this->dashboardService = $dashboardService;
    }

    public function getHoursDeliveredInTheMonthByCurrentUser(Request\Filter $request, Response $response){
        try {
            $hoursDeliveredInTheMonth = $this->dashboardService->getHoursDeliveredInTheMonthByCurrentUser();
            $response->addDataSet(new DataSource\DataSet('hoursDelivered', $hoursDeliveredInTheMonth));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getPlannedHoursByCurrentUser(Request\Filter $request, Response $response){
        try {
            $plannedHours = $this->dashboardService->getPlannedHoursByCurrentUser();
            $response->addDataSet(new DataSource\DataSet('plannedHours', $plannedHours));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getTotalDelayedTasksByCurrentUser(Request\Filter $request, Response $response){
        try {
            $response->addDataSet(new DataSource\DataSet('totalDelayedTasksByCurrentUser', $this->dashboardService->getTotalDelayedTasksByCurrentUser()));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getTotalPendingTasksByCurrentUser(Request\Filter $request, Response $response){
        try {
            $response->addDataSet(new DataSource\DataSet('totalPendingTasksByCurrentUser', $this->dashboardService->getTotalPendingTasksByCurrentUser()));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getTotalDaysWithAppropriationDifference(Request\Filter $request, Response $response){
        try {
            $response->addDataSet(new DataSource\DataSet('totalDaysWithAppropriationDifference',
                $this->dashboardService->getTotalDaysWithAppropriationDifference()));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getTotalIssuesWithPendingConclusion(Request\Filter $request, Response $response){
        try {
            $response->addDataSet(new DataSource\DataSet('totalIssuesWithPendingConclusion',
                $this->dashboardService->getTotalIssuesWithPendingConclusion()));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findTotalBackLogIssuesByCurrentUser(Request\Filter $request, Response $response){
        try {
            Utilities::setDataSetDefault($request,$response,$this->dashboardService->getTotalBackLogIssuesByCurrentUser());
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /******************************************
    ************* Burndown Methods ************
    *******************************************/

    public function fetchWidgetDataInZmart(Request\Filter $request, Response $response)
    {
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $data = $this->dashboardService->fetchWidgetDataInZmart($filters);
            $response->addDataSet(new DataSource\DataSet('fetchWidgetDataInZmart', $data));
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message($ex->getMessage()));
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findOpenSprintByUser(Request\Filter $request, Response $response)
    {
        try {
            $openSprint = $this->dashboardService->findOpenSprintByUser();
            $response->addDataSet(new DataSource\DataSet('findOpenSprintByUser', $openSprint));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findbuildWidgetByViewIdInZmart(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            $drillDownFilters = $row['drillDownFilters'];
            $metaData = $this->dashboardService->findbuildWidgetByViewIdInZmart($drillDownFilters);
            $response->addDataSet(new DataSource\DataSet('findbuildWidgetByViewIdInZmart', $metaData));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findBiggerStructureInZmart(Request\Filter $request, Response $response)
    {
        try {
            $structure = $this->dashboardService->findBiggerStructureInZmart();
            $response->addDataSet(new DataSource\DataSet('findBiggerStructureInZmart', $structure));
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}