<?php
namespace Controller;

use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

class Organization extends Simple {

	/** @var \Service\Appropriation $appropriationService  */
	protected $organizationService;

	public function __construct(\Service\Organization $organizationService) {
		$this->organizationService = $organizationService;
	}

	public function findCurrentOrganization(Request\Filter $request, Response $response) {
		$dataSet = array();
		try{
			$dataSet = $this->organizationService->getCurrentOrganizationData();
			$response->addDataSet(new DataSet('organization', $dataSet));
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message("Não foi possível achar sua organização, pois " . $ex->getMessage()));
		}

	}

	public function saveCurrentOrganization(Request\DataSet $request, Response $response) {

        $organization = $request->getDataSet()->getRows();
        $organization = $organization[0];

		try{
	 		$name = empty($organization['NAME']) ? null : $organization['NAME'];
	 		$foundation = empty($organization['FOUNDATION']) ? null : $organization['FOUNDATION'];
	 		$document = empty($organization['DOCUMENT']) ? null : $organization['DOCUMENT'];
	 		$description = empty($organization['DESCRIPTION']) ? null : $organization['DESCRIPTION'];
            $managers = $organization['MANAGERS'];
            $calendarDefault = $organization['CALENDAR_DEFAULT'];
            $calendar = $organization['CALENDAR'];
            $structure = $organization['STRUCTURE'];
            $timeSheetRequired = empty($organization['TIME_SHEET_REQUIRED']) ? 'N' : $organization['TIME_SHEET_REQUIRED'];
            $replicateTimeSheetRequired = empty($organization['REPLICATE_TIME_SHEET_REQUIRED']) ? false : $organization['REPLICATE_TIME_SHEET_REQUIRED'];
			
			$dataSet = array($this->organizationService->updateCurrentOrganization($name, $foundation, $document,
                $description,$managers,$calendar,$calendarDefault,$structure,$timeSheetRequired,$replicateTimeSheetRequired));

			$response->addNotification(new Response\Notification(\Util\Messages::SUC_SUCCESSFUL_SAVED));
			$response->addDataSet(new DataSet('organization', $dataSet));
		} catch(\Exception $ex){
			$response->addMessage(new Response\Message(\Util\Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
		}

	}
}