<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 23/10/2015
 * Time: 07:33
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Utilities;
use \Zeedhi\DevOps\Model\Projectmanagement\Entities;

class CalendarWeeklyDays
{
    /**
     * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\CalendarWeeklyDays
     */
    protected $calendarWeeklyDaysRepository;
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var Utilities
     */
    private $environment;
    /**
     * @var \Factory\CalendarWeeklyDays
     */
    private $calendarWeeklyDaysFactory;
    /**
     * @var CalendarDates
     */
    private $calendarDatesService;
    /**
     * @var \Factory\CalendarDetail
     */
    private $calendarDetailFactory;

    public function __construct(EntityManager $entityManager, Utilities $environment,
                                \Factory\CalendarWeeklyDays $calendarWeeklyDaysFactory,
                                CalendarDates $calendarDatesService, \Factory\CalendarDetail $calendarDetailFactory)
    {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->calendarWeeklyDaysRepository = $entityManager
            ->getRepository(Utilities::REPOSITORY_DIR . 'CalendarWeeklyDays');
        $this->calendarWeeklyDaysFactory = $calendarWeeklyDaysFactory;
        $this->calendarDatesService = $calendarDatesService;
        $this->calendarDetailFactory = $calendarDetailFactory;
    }

    /**
     * @param $typeSunday
     * @param $dailyHoursSunday
     * @param $typeMonday
     * @param $dailyHoursMonday
     * @param $typeTuesday
     * @param $dailyHoursTuesday
     * @param $typeWednesday
     * @param $dailyHoursWednesday
     * @param $typeThursday
     * @param $dailyHoursThursday
     * @param $typeFriday
     * @param $dailyHoursFriday
     * @param $typeSaturday
     * @param $dailyHoursSaturday
     * @return array
     */
    public static function createArrayWorkWeek($typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday, $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday)
    {
        $week = array(
            array($dailyHoursSunday, $typeSunday),
            array($dailyHoursMonday, $typeMonday),
            array($dailyHoursTuesday, $typeTuesday),
            array($dailyHoursWednesday, $typeWednesday),
            array($dailyHoursThursday, $typeThursday),
            array($dailyHoursFriday, $typeFriday),
            array($dailyHoursSaturday, $typeSaturday)
        );
        return $week;
    }

    public function saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday, $typeMonday,
                                   $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday,
                                   $dailyHoursWednesday, $typeThursday, $dailyHoursThursday, $typeFriday,
                                   $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $calendar = $this->entityManager->find(Utilities::ENTITY_DIR.'Calendar', $calendar);
            $calendarDetail = $this->editWeekWorkBase($id, $calendar, $name, $beginDate, $endDate);
            $this->createWeeklyDays($typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday, $typeTuesday,
                $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday, $dailyHoursThursday,
                $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $calendarDetail);
            $this->entityManager->flush();
            $this->calendarDatesService->setWorkWeek($calendarDetail);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $calendarDetail;
    }

    public function removeAllWeeklyDays($calendarDetail, $flush = false)
    {
        try {
            $calendarWeeklyDays = $this->calendarWeeklyDaysRepository->findBy(array('calendarDetail' => $calendarDetail));
            $this->calendarDatesService->removeAllDatesByCalendarDetail($calendarDetail);
            foreach ($calendarWeeklyDays as $calendarWeeklyDay) {
                $this->entityManager->remove($calendarWeeklyDay);
            }
            if ($flush) {
                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function copyAllCalendarWeeklyDaysService(Entities\CalendarDetail $calendarDetail,Entities\CalendarDetail $calendarDetailClone)
    {
        $connection = $this->entityManager->getConnection();
        try {
            /** @var Entities\CalendarWeeklyDays[] $calendarWeeklyDays */
            $calendarWeeklyDays = $this->calendarWeeklyDaysRepository->findBy(array('calendarDetail' => $calendarDetail));
            foreach ($calendarWeeklyDays as $calendarWeeklyDay) {
                $calendarWeeklyDayClone = clone $calendarWeeklyDay;
                $calendarWeeklyDayClone->setCalendarDetail($calendarDetailClone);
                $this->entityManager->persist($calendarWeeklyDayClone);
            }
            $this->entityManager->flush();
            $this->calendarDatesService->setDatesByCalendarDetail($calendarDetailClone);
            $this->entityManager->flush();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function findCalendarWeeklyDays($calendarId, $id, $withBase = true, $filter = '' )
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar $calendar */
        $calendar = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar', $calendarId);
        $calendarWeeklyDays = $this->calendarWeeklyDaysRepository->getCalendarWeeklyDays($calendarId, $id, $organization,$withBase, $filter);
        foreach ($calendarWeeklyDays as &$calendarWeeklyDay) {
            if($calendarWeeklyDay['ORIGIN_CALENDAR_DETAIL']){
                $calendarWeeklyDay['TO_CALENDAR'] = 'Não';
                $calendarWeeklyDay['EDIT'] = false;
            }else{
                $calendarWeeklyDay['TO_CALENDAR'] = 'Sim';
                $calendarWeeklyDay['EDIT'] = true;
            }
        }
        if ($calendar && $calendar->getCalendarDefault()) {
            $defaultCalendarWeeklyDays = $this->calendarWeeklyDaysRepository->getCalendarWeeklyDays(
                $calendar->getCalendarDefault()->getId(), $id, $organization,$withBase, $filter);
            foreach ($defaultCalendarWeeklyDays as &$defaultCalendarWeeklyDay) {
                $defaultCalendarWeeklyDay['TO_CALENDAR'] = 'Não';
                $defaultCalendarWeeklyDay['EDIT'] = false;
            }
            $calendarWeeklyDays = array_merge($calendarWeeklyDays, $defaultCalendarWeeklyDays);
        }
        return $calendarWeeklyDays;
    }
    public function findCalendarWeeklyDaysStructure($structureId, $id){
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
        $structure = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Structure')->find($structureId);
        $calendar = Utilities::getNextCalendar($structure);
        if($calendar){
            $calendarId = $calendar->getId();
        }else{
            $calendarId = 0;
        }
        $calendarWeeklyDays = $this->findCalendarWeeklyDays($calendarId, $id);
        if(!$structure->getCalendar() || $calendar->getId() != $structure->getCalendar()->getId()){
            foreach($calendarWeeklyDays as &$calendarWeeklyDay){
                $calendarWeeklyDay['TO_CALENDAR'] = 'Não';
                $calendarWeeklyDay['EDIT'] = false;
            }
        }
        return $calendarWeeklyDays;
    }
    public function findCalendarWeeklyDaysByResource(Entities\Resource $resource, Entities\Structure $structure = null,$id){

        $calendarWeeklyDays = array();
        $structureCalendar = null;
        if($structure){
            $structureCalendar = Utilities::getNextCalendar($structure);
        }
        if($structureCalendar) {
            $calendarWeeklyDays = $this->findCalendarWeeklyDays($structureCalendar->getId(), $id,!$resource->getCalendar());
            foreach ($calendarWeeklyDays as &$calendarException) {
                $calendarException['TO_CALENDAR'] = 'Não';
                $calendarException['EDIT'] = false;
            }
        }
        if($resource->getCalendar()){
            $calendarWeeklyDays = array_merge($this->findCalendarWeeklyDays($resource->getCalendar()->getId(), $id),$calendarWeeklyDays);
            $calendarWeeklyDays = Utilities::getUniqueArrayMultidimensional($calendarWeeklyDays); ;
        }
        return $calendarWeeklyDays;
    }
    public function findCalendarWeeklyDaysByProject(Entities\Project $project, Entities\Structure $structure,$id, $filter = ''){

        $calendarWeeklyDays = array();
        $structureCalendar = Utilities::getNextCalendar($structure);
        if($structureCalendar) {
            $calendarWeeklyDays = $this->findCalendarWeeklyDays($structureCalendar->getId(), $id, true,$filter);
            foreach ($calendarWeeklyDays as &$calendarException) {
                $calendarException['TO_CALENDAR'] = 'Não';
                $calendarException['EDIT'] = false;
            }
        }
        if($project->getCalendar()){
            $calendarWeeklyDays = array_merge($this->findCalendarWeeklyDays($project->getCalendar()->getId(), $id, true, $filter),$calendarWeeklyDays);
            $calendarWeeklyDays = Utilities::getUniqueArrayMultidimensional($calendarWeeklyDays); ;
        }
        return $calendarWeeklyDays;
    }
    public function findCalendarWeeklyDaysByProjectResource(Entities\ResourceProject $projectResource ,$id)
    {
        $project = $projectResource->getProject();
        $resource = $projectResource->getResourceC();
        $calendarWeeklyDays = $this->findCalendarWeeklyDaysByProject($project, $project->getStructure(), $id);
        $structure = $this->environment->getStructureForResource($resource);
        $calendarWeeklyDays = array_merge($this->findCalendarWeeklyDaysByResource($resource, $structure, $id), $calendarWeeklyDays);

        foreach ($calendarWeeklyDays as &$calendarException) {
            $calendarException['TO_CALENDAR'] = 'Não';
            $calendarException['EDIT'] = false;
        }

        if ($projectResource->getCalendar()) {
            $calendarWeeklyDays = array_merge($this->findCalendarWeeklyDays($projectResource->getCalendar()->getId(), $id), $calendarWeeklyDays);
        }
        $calendarWeeklyDays = Utilities::getUniqueArrayMultidimensional($calendarWeeklyDays);;
        return $calendarWeeklyDays;
    }

    /**
     * @param $typeSunday
     * @param $dailyHoursSunday
     * @param $typeMonday
     * @param $dailyHoursMonday
     * @param $typeTuesday
     * @param $dailyHoursTuesday
     * @param $typeWednesday
     * @param $dailyHoursWednesday
     * @param $typeThursday
     * @param $dailyHoursThursday
     * @param $typeFriday
     * @param $dailyHoursFriday
     * @param $typeSaturday
     * @param $dailyHoursSaturday
     * @param $calendarDetail
     * @throws \Exception
     */
    public function createWeeklyDays($typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday, $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $calendarDetail)
    {
        $this->removeAllWeeklyDays($calendarDetail, true);
        $week = self::createArrayWorkWeek($typeSunday, $dailyHoursSunday, $typeMonday, $dailyHoursMonday,
            $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday, $dailyHoursThursday,
            $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday);
        foreach ($week as $key => $weekDay) {
            $hours = $weekDay[0];
            $type = $weekDay[1];
            if ($type == CalendarDetail::OUT_DAY_TYPE) {
                $hours = 0;
            } else {
                $hours = Utilities::timeToSeconds($hours);
            }
            $calendarWeekDays = $this->calendarWeeklyDaysFactory->createCalendarWeeklyDay($calendarDetail, $key + 1, $hours, $type);
            $this->entityManager->persist($calendarWeekDays);
        }
    }

    /**
     * @param $id
     * @param $calendar
     * @param $name
     * @param $beginDate
     * @param $endDate
     * @return Entities\CalendarDetail
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     * @throws \Doctrine\ORM\TransactionRequiredException
     */
    public function editWeekWorkBase($id, $calendar, $name, $beginDate, $endDate)
    {
        if (empty($id)) {
            $calendarDetail = $this->calendarDetailFactory->createWeekWorkBase($calendar, $name, $beginDate, $endDate);
            $this->entityManager->persist($calendarDetail);
            $this->entityManager->flush();
            return $calendarDetail;
        } else {
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail $calendarDetail */
            $calendarDetail = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDetail', $id);
            $calendarDetail = $this->calendarDetailFactory->updateWeekWorkBase($calendarDetail, $calendar, $name, $beginDate,
                $endDate);
            return $calendarDetail;
        }
    }
}