<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class ReplanningReason
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    
    public function __construct(EntityManager $entityManager)
    {
         $this->entityManager = $entityManager;
    }
    
    public function findReplanningReason($conditions)
    {
        $replanningReasonRepository = InstanceProvider::getRepository()->getReplanningReason();
        $environment = InstanceProvider::getEnvironment();
        $alias = 'RPR';
        foreach($conditions as &$condition){
            if($condition['columnName']  == 'STATUS_FILTER'){
                $condition['columnName'] = 'STATUS';
            }
        }
        $filters = DevOpsUtilities::createQueryFiltered($conditions, $alias);
        $organizationId = $environment->getCurrentOrganizationId();
        $replanningReason = $replanningReasonRepository->findReplanningReason($organizationId, $filters);
        return $replanningReason;
    }
    
    public function saveReplanningReason($id, $description, $status)
    {
        $connection = $this->entityManager->getConnection();
        $replanningReasonRepository = InstanceProvider::getRepository()->getReplanningReason();
        $replanningReasonFactory = InstanceProvider::getFactory()->getReplanningReason();
        $replanningReasonEntity = NULL;
        try {
            $connection->beginTransaction();
            if($id){
                $replanningReasonEntity = $replanningReasonRepository->find($id);
            }
            $replanningReasonEntity = $replanningReasonFactory
                                         ->createReplanningReason($replanningReasonEntity, $description, $status);
            $this->entityManager->persist($replanningReasonEntity);
            $this->entityManager->flush();
            $connection->commit();
            return array("ID" => $replanningReasonEntity->getId());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteReplanningReason($replanningReasonId)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $replanningReasonRepository = InstanceProvider::getRepository()->getReplanningReason();
            $replanningReasonEntity = $replanningReasonRepository->find($replanningReasonId);
            $replanningReasonEntity->setStatus(Entities\ReplanningReason::DELETED);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Doctrine\DBAL\DBALException $e) {
            $connection->rollBack();
            throw new \Exception($e->getMessage(), 1);
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
}