<?php

namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Deploy extends \Doctrine\ORM\EntityRepository
{
    /**
     * @param $productId
     * @param string $filters
     * @return array
     */

    public function getMaxDeployDateByServers($product, $servers)
    {
        $query = <<<SQL
            SELECT TO_CHAR(MAX(DY.DEPLOY_DATE), 'DD/MM/YYYY HH24:MI:SS') AS MAX_DATE
              FROM DEPLOY DY
        INNER JOIN DEPLOY_SERVER    DS ON DY.ID         = DS.DEPLOY_ID
        INNER JOIN PRODUCT_SERVER   PS ON PS.ID         = DS.PRODUCT_SERVER_ID
        INNER JOIN SERVER           SR ON PS.SERVER_ID  = SR.ID
             WHERE DY. PRODUCT_ID = :PRODUCT
               AND PS.ID IN ($servers)
SQL;
        $params = array(
            'PRODUCT' => $product
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function getDeploy($productId, $filters = "", $organizationId, $limit = null, $offset = null)
    {
        $query = <<<SQL
			SELECT DP.ID, DP.NAME, DP.PRODUCT_ID, DP.PRODUCT_VERSION_ID, DP.VERSION, DP.PRODUCT_NAME,
                   DP.IS_ACTIVE, DP.STATUS, DP.PATH_BACKUP, TO_CHAR(DP.DEPLOY_DATE, 'DD/MM/YYYY HH24:MI:SS') AS DEPLOY_DATE,
                   DP.IS_CUSTOMER_DEPLOY, DP.CREATED_BY, DP.CREATED, DP.UPDATED_BY, DP.UPDATED,
                   (
                      SELECT rtrim(xmlagg(xmlelement(e,PS.ID,',').extract('//text()') order by SR.ID).GetClobVal(),',')
                       AS SERVER
                        FROM DEPLOY_SERVER DS
                       INNER JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                       INNER JOIN SERVER SR ON PS.SERVER_ID = SR.ID
                       WHERE DS.DEPLOY_ID = DP.ID
                         AND PS.PRODUCT_ID = DP.PRODUCT_ID
                         AND PS.USETYPE = 'P'
                         AND PS.ORGANIZATION_ID = :ORGANIZATION
                   ) AS SERVER,
                   (
                      SELECT rtrim(xmlagg(xmlelement(e,PC.ID,',').extract('//text()') order by PC.ID).GetClobVal(),',')
                      AS SERVER
                        FROM DEPLOY_PACKAGE DC
                       INNER JOIN PRODUCT_VERSION_PACKAGES PC ON DC.PRODUCT_VERSION_PACKAGE_ID = PC.ID
                       INNER JOIN PRODUCT_VERSION PV ON PV.ID = PC.PRODUCT_VERSION_ID
                       WHERE DC.DEPLOY_ID = DP.ID
                         AND PV.ID = DP.PRODUCT_VERSION_ID
                   ) AS PACKAGE
              FROM ( SELECT *
                       FROM (
                                SELECT DP.ID, DP.NAME, DP.PRODUCT_ID, DP.PRODUCT_VERSION_ID, DP.IS_ACTIVE, PR.NAME AS PRODUCT_NAME,
                                       DECODE(PV.VERSION, NULL, '-', PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD) AS VERSION,
                                       DP.DEPLOY_DATE, DP.STATUS, DP.PATH_BACKUP, ZC.ID AS CREATED_BY_ID, ZC.NAME AS CREATED_BY, DP.CREATED,
                                       ZU.ID AS UPDATED_BY_ID, ZU.NAME AS UPDATED_BY, DP.UPDATED,
                                       SR.ID AS SERVER, SR.NAME AS SERVER_NAMES, PS.ID AS PRODUCT_SERVER_ID, DP.IS_CUSTOMER_DEPLOY
                                  FROM DEPLOY DP
                                  LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DP.PRODUCT_VERSION_ID
                                  LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
                                  LEFT JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                                  LEFT JOIN SERVER SR ON PS.SERVER_ID = SR.ID
                                  LEFT JOIN PRODUCT PR ON DP.PRODUCT_ID = PR.ID
                                  LEFT JOIN ZH_USER ZC ON ZC.ID = DP.CREATED_BY
                                  LEFT JOIN ZH_USER ZU ON ZU.ID = DP.UPDATED_BY
                                 WHERE DP.PRODUCT_ID = :PRODUCT
                                 AND DP.IS_CUSTOMER_DEPLOY = 'N'
                            )
                      WHERE 1=1
                   $filters
                   ) DP
             GROUP BY DP.ID, DP.NAME, DP.PRODUCT_ID, DP.PRODUCT_VERSION_ID, DP.DEPLOY_DATE, DP.VERSION, DP.IS_ACTIVE,
                      DP.STATUS, DP.PATH_BACKUP, DP.PRODUCT_NAME, DP.IS_CUSTOMER_DEPLOY, DP.CREATED_BY, DP.CREATED, DP.UPDATED_BY, DP.UPDATED
             ORDER BY DP.DEPLOY_DATE DESC,  DP.ID DESC
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'ORGANIZATION' => $organizationId,
        );
        $query = DevOpsUtilities::getPaginatedQuery($query, $limit, $offset);
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCustomerDeploys($productId, $customerCode, $filterQuery)
    {

        $query = <<<SQL
            SELECT *
              FROM (SELECT DP.ID, DP.NAME, DP.PRODUCT_ID, DP.PRODUCT_VERSION_ID, DP.IS_ACTIVE, PR.NAME AS PRODUCT_NAME,
                           DECODE(PV.VERSION, NULL, '-', PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD) AS VERSION,
                           DECODE(PP.VERSION, NULL, '-', PP.VERSION || '.' || PP.REVISION  || '.' || PP.BUILD) AS PREVIOUS_VERSION,
                           DP.DEPLOY_DATE, DP.PATH_BACKUP, DP.PARENT, DP.TICKET_ID,DP.CD_SYSTEM_TICKET, PR.URL_WEBSERVICE, SR.WSEMAIL,
                           SR.ID AS SERVER, NVL(SR.NAME, '-') AS SERVER_NAME, PS.ID AS PRODUCT_SERVER_ID,
                           NVL(PS.PATH, '-') AS BASE_PATH, SR.IS_ACTIVE AS SERVER_IS_ACTIVE,
                           SR.IS_WINDOWS AS SERVER_IS_WINDOWS, OPC.NMOPERADOR AS CREATED_BY, DP.CREATED,
                           OPU.NMOPERADOR AS UPDATED_BY, DP.UPDATED,
                           CASE WHEN PV.ID = PP.ID AND DP.STATUS = 'D'
                                THEN 'I'
                                ELSE DP.STATUS
                            END AS STATUS
                      FROM (SELECT D.ID
                              FROM DEPLOY D
                             WHERE D.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
                             START WITH D.PRODUCT_ID = :PRODUCT
                            CONNECT BY PRIOR D.ID = D.PARENT) DT
                     INNER JOIN DEPLOY DP ON DP.ID = DT.ID
                      LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DP.PRODUCT_VERSION_ID
                      LEFT JOIN PRODUCT_VERSION PP ON PP.ID = DP.PREVIOUS_PRODUCT_VERSION_ID
                      LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
                      LEFT JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                      LEFT JOIN SERVER SR ON PS.SERVER_ID = SR.ID
                      LEFT JOIN PRODUCT PR ON DP.PRODUCT_ID = PR.ID
                      INNER JOIN RESOURCE_T RT ON RT.ID = DP.RESOURCE_T_ID
                                             AND RT.EXTERNAL_CODE = :EXTERNAL_CODE

                      LEFT JOIN ZH_USER ZC ON ZC.ID = DP.CREATED_BY
                      LEFT JOIN RESOURCE_T RTC ON RTC.ID = ZC.EXTERNAL_RES_ID
                      LEFT JOIN OPERADOR OPC ON OPC.CDOPERADOR = RTC.EXTERNAL_CODE

                      LEFT JOIN ZH_USER ZU ON ZU.ID = DP.UPDATED_BY
                      LEFT JOIN RESOURCE_T RTU ON RTU.ID = ZU.EXTERNAL_RES_ID
                      LEFT JOIN OPERADOR OPU ON OPU.CDOPERADOR = RTU.EXTERNAL_CODE

                     WHERE DP.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
                       AND DP.IS_DEPLOY_STEP = 'N'
                  ORDER BY DP.DEPLOY_DATE DESC)
             WHERE 1 = 1
             $filterQuery
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'EXTERNAL_CODE' => $customerCode,
            'IS_CUSTOMER_DEPLOY' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::IS_CUSTOMER_DEPLOY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findCustomerDeploySteps($deployStepId)
    {
        $query = <<<SQL
            SELECT DP.ID, DP.NAME, DP.PRODUCT_ID, DP.PRODUCT_VERSION_ID, DP.IS_ACTIVE, PR.NAME AS PRODUCT_NAME,
                   DECODE(PV.VERSION, NULL, '-', PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD) AS VERSION,
                   DECODE(PP.VERSION, NULL, '-', PP.VERSION || '.' || PP.REVISION  || '.' || PP.BUILD) AS PREVIOUS_VERSION,
                   DP.DEPLOY_DATE, DP.PATH_BACKUP, DP.PARENT, DP.TICKET_ID, PR.URL_WEBSERVICE, SR.WSEMAIL,
                   CASE WHEN PV.ID = PP.ID AND DP.STATUS = 'D'
                        THEN 'I'
                        ELSE DP.STATUS
                    END AS STATUS,
                   SR.ID AS SERVER, NVL(SR.NAME, '-') AS SERVER_NAME, PS.ID AS PRODUCT_SERVER_ID,
                   NVL(PS.PATH, '-') AS BASE_PATH, SR.IS_ACTIVE AS SERVER_IS_ACTIVE,
                   SR.IS_WINDOWS AS SERVER_IS_WINDOWS, DR.SERVER_ORDER,
                   OPC.NMOPERADOR AS CREATED_BY, DP.CREATED, OPU.NMOPERADOR AS UPDATED_BY, DP.UPDATED
              FROM (SELECT D.ID
                      FROM DEPLOY D
                     WHERE D.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
                START WITH D.ID IN (SELECT DPL.ID
                                      FROM DEPLOY_STEP_SERVER DSS
                                      JOIN DEPLOY DPL ON DPL.DEPLOY_STEP_SRV_ID = DSS.ID
                                     WHERE DSS.DEPLOY_STEP_ID = :DEPLOY_STEP_ID)
                   CONNECT BY PRIOR D.ID = D.PARENT) DT
             INNER JOIN DEPLOY DP ON DP.ID = DT.ID
              LEFT JOIN DEPLOY_STEP_SERVER DR ON DR.ID = DP.DEPLOY_STEP_SRV_ID
              LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DP.PRODUCT_VERSION_ID
              LEFT JOIN PRODUCT_VERSION PP ON PP.ID = DP.PREVIOUS_PRODUCT_VERSION_ID
              LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
              LEFT JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
              LEFT JOIN SERVER SR ON PS.SERVER_ID = SR.ID
              LEFT JOIN PRODUCT PR ON DP.PRODUCT_ID = PR.ID

              LEFT JOIN ZH_USER ZC ON ZC.ID = DP.CREATED_BY
              LEFT JOIN RESOURCE_T RTC ON RTC.ID = ZC.EXTERNAL_RES_ID
              LEFT JOIN OPERADOR OPC ON OPC.CDOPERADOR = RTC.EXTERNAL_CODE

              LEFT JOIN ZH_USER ZU ON ZU.ID = DP.UPDATED_BY
              LEFT JOIN RESOURCE_T RTU ON RTU.ID = ZU.EXTERNAL_RES_ID
              LEFT JOIN OPERADOR OPU ON OPU.CDOPERADOR = RTU.EXTERNAL_CODE

             WHERE DP.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
               AND DP.IS_DEPLOY_STEP = 'Y'
          ORDER BY DR.SERVER_ORDER, DP.DEPLOY_DATE NULLS LAST
SQL;
        $params = array(
            'DEPLOY_STEP_ID' => $deployStepId,
            'IS_CUSTOMER_DEPLOY' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::IS_CUSTOMER_DEPLOY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function findDeployAudit($productId, $serverId, $startDate, $endDate, $isClient, $cdCliente)
    {
        $query = <<<SQL
        SELECT  *
          FROM (SELECT DISTINCT
                       CP.NRCALL AS ID, D.DEPLOY_ID, D.DEPLOY_DATE, D.PRODUCT_VERSION_ID,
                       D.VERSION AS PV_VERSION, D.REVISION AS PV_REVISION, D.BUILD AS PV_BUILD,
                       D.PRODUCT_NAME, D.PRODUCT_ID, ZHU.NAME AS OPERATOR_NAME,
                       DECODE(D.VERSION, NULL, '-', LPAD(D.VERSION, 4, ' ') || '.' || LPAD(D.REVISION, 4, ' ')  || '.' || LPAD(D.BUILD, 4, ' ')) AS VERSION,
                       DECODE(I.ID, NULL, T.ID, I.ID) AS TEKNISA_ID, DECODE(I.IS_ISSUE, NULL, T.IS_ISSUE, I.IS_ISSUE) AS IS_ISSUE,
                       DECODE(I.NAME, NULL, T.NAME, I.NAME)  AS NAME,
                       DECODE(I.RESPONSE, NULL, T.RESPONSE, I.RESPONSE) AS RESPONSE,
                       DECODE(SRV.NAME, NULL, SRP.NAME, SRV.NAME) AS SERVER_NAME,
                       DECODE(PDS.PATH, NULL, PDP.PATH, PDS.PATH) AS BASE_PATH,
                       DECODE(RI.EXTERNAL_CODE, :CDCLIENTE, 'S', 'N') AS IS_CLIENT,
                       CASE WHEN D.PREVIOUS_PRODUCT_VERSION_ID IS NULL
                            THEN 'N'
                            WHEN TO_NUMBER(LPAD(D.VERSION, 5, '0') || LPAD(D.REVISION, 5, '0') || LPAD(D.BUILD, 5, '0'))
                                 <
                                 TO_NUMBER(LPAD(D.PREVIOUS_VERSION, 5, '0') || LPAD(D.PREVIOUS_REVISION, 5, '0') || LPAD(D.PREVIOUS_BUILD, 5, '0'))
                            THEN 'S'
                            ELSE 'N'
                        END AS DOWNGRADE
                  FROM (SELECT DP.UPDATED_BY, DP.ID AS DEPLOY_ID, DP.DEPLOY_DATE, DP.ID, DP.NAME, DP.PRODUCT_ID,
                               P.VERSION, P.REVISION, P.BUILD, P.ID AS PRODUCT_VERSION_ID, DP.PARENT,
                               PR.NAME AS PRODUCT_NAME, PV.ID AS PREVIOUS_PRODUCT_VERSION_ID,
                               PV.VERSION AS PREVIOUS_VERSION, PV.REVISION AS PREVIOUS_REVISION, PV.BUILD AS PREVIOUS_BUILD
                          FROM DEPLOY DP
                          JOIN PRODUCT_VERSION P ON P.ID = DP.PRODUCT_VERSION_ID
                     LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DP.PREVIOUS_PRODUCT_VERSION_ID
                          JOIN PRODUCT PR ON DP.PRODUCT_ID = PR.ID
                         WHERE DP.ID IN (SELECT DP.ID
                                           FROM DEPLOY DP
                                           JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
                                           JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                                          WHERE DP.DEPLOY_DATE BETWEEN TO_DATE(:INITIALDATE, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:FINALDATE, 'DD/MM/YYYY HH24:MI:SS')
                                            AND DP.PRODUCT_ID = :PRODUCT
                                            AND PS.ID = :SRV_ID
                                            AND DP.IS_CUSTOMER_DEPLOY = 'Y'
                                            AND DP.STATUS = 'D'
                                          UNION
                                         SELECT DP.ID
                                           FROM DEPLOY DP
                                          WHERE DP.PARENT IN (SELECT DP.ID
                                                                FROM DEPLOY DP
                                                                JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
                                                                JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                                                               WHERE DP.DEPLOY_DATE BETWEEN TO_DATE(:INITIALDATE, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:FINALDATE, 'DD/MM/YYYY HH24:MI:SS')
                                                                 AND DP.PRODUCT_ID = :PRODUCT
                                                                 AND PS.ID = :SRV_ID
                                                                 AND DP.IS_CUSTOMER_DEPLOY = 'Y'
                                                                 AND DP.STATUS = 'D'))
                       ) D
                  JOIN RELEASE R ON R.PRODUCT_ID = D.PRODUCT_ID
                  JOIN SUBRELEASE S ON S.RELEASE_ID = R.ID
                                   AND S.VERSION  = D.VERSION
                                   AND S.REVISION = D.REVISION
                                   AND S.BUILD    = D.BUILD
                  JOIN BRANCH B ON B.SUBRELEASE_ID = S.ID
                  JOIN TASK_BRANCHES TB ON TB.NAME = B.NAME
                  JOIN TASK T ON TB.TASK = T.ID
                  JOIN CLASSIFICATION C ON C.ID = T.CLASSIFICATION AND (C.USED_IN_NEWS = 'Y' OR C.CONTROL IS NULL)
                  JOIN ZH_USER ZHU ON D.UPDATED_BY = ZHU.ID
             LEFT JOIN TASK_RELATIONSHIP TR ON TR.TASK_RELATIONSHIP = T.ID AND TR.TYPE = 'I'
             LEFT JOIN TASK I ON I.ID = TR.TASK
             LEFT JOIN CP_CALL CP ON I.ID = CP.CALL_RELATIONSHIP

             LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = D.ID
             LEFT JOIN PRODUCT_SERVER PDS ON PDS.ID = DS.PRODUCT_SERVER_ID
             LEFT JOIN SERVER SRV ON PDS.SERVER_ID = SRV.ID

             LEFT JOIN DEPLOY_SERVER DSP ON DSP.DEPLOY_ID = D.PARENT
             LEFT JOIN PRODUCT_SERVER PDP ON PDP.ID = DSP.PRODUCT_SERVER_ID
             LEFT JOIN SERVER SRP ON PDP.SERVER_ID = SRP.ID

             LEFT JOIN ALLOCATION A ON A.TASK = I.ID AND A.TYPE = 'S'
             LEFT JOIN RESOURCE_T RI ON RI.ID = A.RESOURCE_T)
         WHERE IS_CLIENT  = :IS_CLIENT
      ORDER BY TO_NUMBER(LPAD(PV_VERSION, 5, '0') || LPAD(PV_REVISION, 5, '0') || LPAD(PV_BUILD, 5, '0')), DEPLOY_DATE, NAME
SQL;
        $params = array(
            'SRV_ID'      => $serverId,
            'INITIALDATE' => $startDate,
            'FINALDATE'   => $endDate,
            'PRODUCT'     => $productId,
            'IS_CLIENT'   => $isClient,
            'CDCLIENTE'   => $cdCliente
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findVersionAudit($productId, $moduleId, $isClient, $cdCliente,  $initialVersion, $finalVersion, $initialRevision,$finalRevision,$initialBuild,$finalBuild)
    {
        $query = <<<SQL
             SELECT *
               FROM (SELECT DISTINCT
                            CP.NRCALL AS ID, D.PRODUCT_VERSION_ID,
                            D.VERSION AS PV_VERSION, D.REVISION AS PV_REVISION, D.BUILD AS PV_BUILD,
                            D.PRODUCT_NAME, D.PRODUCT_ID, D.DEPLOY_DATE, ZHU.NAME AS OPERATOR_NAME,
                            DECODE(I.ID, NULL, T.ID, I.ID) AS TEKNISA_ID, DECODE(I.IS_ISSUE, NULL, T.IS_ISSUE, I.IS_ISSUE) AS IS_ISSUE,
                            DECODE(D.VERSION, NULL, '-', LPAD(D.VERSION, 4, ' ') || '.' || LPAD(D.REVISION, 4, ' ')  || '.' || LPAD(D.BUILD, 4, ' ')) AS VERSION,
                            DECODE(I.NAME, NULL, T.NAME, I.NAME)  AS NAME,
                            DECODE(I.RESPONSE, NULL, T.RESPONSE, I.RESPONSE) AS RESPONSE, D.ID AS DEPLOY_ID,
                            DECODE(SRV.NAME, NULL, SRP.NAME, SRV.NAME) AS SERVER_NAME,
                            DECODE(PDS.PATH, NULL, PDP.PATH, PDS.PATH) AS BASE_PATH,
                            DECODE(RI.EXTERNAL_CODE, :CDCLIENTE, 'S', 'N') AS IS_CLIENT,
                            CASE WHEN D.PREVIOUS_PRODUCT_VERSION_ID IS NULL
                                 THEN 'N'
                                 WHEN TO_NUMBER(LPAD(D.VERSION, 5, '0') || LPAD(D.REVISION, 5, '0') || LPAD(D.BUILD, 5, '0'))
                                      <
                                      TO_NUMBER(LPAD(D.PREVIOUS_VERSION, 5, '0') || LPAD(D.PREVIOUS_REVISION, 5, '0') || LPAD(D.PREVIOUS_BUILD, 5, '0'))
                                 THEN 'S'
                                 ELSE 'N'
                             END AS DOWNGRADE
                      FROM (SELECT D.ID, D.NAME, D.PRODUCT_ID, D.PARENT, DP.PRODUCT_ID AS D_PRODUCT_ID,
                                   P.VERSION, P.REVISION, P.BUILD, P.ID AS PRODUCT_VERSION_ID,
                                   PR.NAME AS PRODUCT_NAME, D.DEPLOY_DATE, D.UPDATED_BY,
                                   PV.VERSION AS PREVIOUS_VERSION, PV.REVISION AS PREVIOUS_REVISION, PV.BUILD AS PREVIOUS_BUILD, PV.ID AS PREVIOUS_PRODUCT_VERSION_ID
                              FROM DEPLOY D
                         LEFT JOIN DEPLOY DP ON DP.ID = D.PARENT
                              JOIN PRODUCT_VERSION P ON P.ID = D.PRODUCT_VERSION_ID
                         LEFT JOIN PRODUCT_VERSION PV ON PV.ID = D.PREVIOUS_PRODUCT_VERSION_ID
                              JOIN PRODUCT PR ON D.PRODUCT_ID = PR.ID
                             WHERE PR.ID = :MODULE_ID
                               AND (D.PARENT IS NULL OR DP.PRODUCT_ID = :MAIN_PRODUCT_ID)
                               AND D.IS_CUSTOMER_DEPLOY = 'Y'
                               AND D.STATUS = 'D'
                               AND TO_NUMBER(LPAD(P.VERSION, 5, '0')     || LPAD(P.REVISION, 5, '0')     || LPAD(P.BUILD, 5, '0')) BETWEEN
                                   TO_NUMBER(LPAD(:INITVERSION, 5, '0')  || LPAD(:INITREVISION, 5, '0')  || LPAD(:INITBUILD, 5, '0')) AND
                                   TO_NUMBER(LPAD(:FINALVERSION, 5, '0') || LPAD(:FINALREVISION, 5, '0') || LPAD(:FINALBUILD, 5, '0'))
                             ) D
                      JOIN RELEASE R ON R.PRODUCT_ID = D.PRODUCT_ID
                      JOIN SUBRELEASE S ON S.RELEASE_ID = R.ID
                                       AND S.VERSION  = D.VERSION
                                       AND S.REVISION = D.REVISION
                                       AND S.BUILD    = D.BUILD
                      JOIN BRANCH B ON B.SUBRELEASE_ID = S.ID
                      JOIN TASK_BRANCHES TB ON TB.NAME = B.NAME
                      JOIN TASK T ON TB.TASK = T.ID
                      JOIN CLASSIFICATION C ON C.ID = T.CLASSIFICATION AND (C.USED_IN_NEWS = 'Y' OR C.CONTROL IS NULL)
                      JOIN ZH_USER ZHU ON D.UPDATED_BY = ZHU.ID
                 LEFT JOIN TASK_RELATIONSHIP TR ON TR.TASK_RELATIONSHIP = T.ID AND TR.TYPE = 'I'
                 LEFT JOIN TASK I ON I.ID = TR.TASK
                 LEFT JOIN CP_CALL CP ON I.ID = CP.CALL_RELATIONSHIP AND CP.TYPE = 'I'

                 LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = D.ID
                 LEFT JOIN PRODUCT_SERVER PDS ON PDS.ID = DS.PRODUCT_SERVER_ID
                 LEFT JOIN SERVER SRV ON PDS.SERVER_ID = SRV.ID

                 LEFT JOIN DEPLOY_SERVER DSP ON DSP.DEPLOY_ID = D.PARENT
                 LEFT JOIN PRODUCT_SERVER PDP ON PDP.ID = DSP.PRODUCT_SERVER_ID
                 LEFT JOIN SERVER SRP ON PDP.SERVER_ID = SRP.ID

                 LEFT JOIN ALLOCATION A ON A.TASK = I.ID AND A.TYPE = 'S'
                 LEFT JOIN RESOURCE_T RI ON RI.ID = A.RESOURCE_T)
              WHERE IS_CLIENT = :IS_CLIENT
           ORDER BY TO_NUMBER(LPAD(PV_VERSION, 5, '0') || LPAD(PV_REVISION, 5, '0') || LPAD(PV_BUILD, 5, '0')), DEPLOY_DATE, NAME
SQL;
        $params = array(
            'MODULE_ID'       => $moduleId,
            'MAIN_PRODUCT_ID' => $productId,
            'INITVERSION'     => $initialVersion,
            'FINALVERSION'    => $finalVersion,
            'INITREVISION'    => $initialRevision,
            'FINALREVISION'   => $finalRevision,
            'INITBUILD'       => $initialBuild,
            'FINALBUILD'      => $finalBuild,
            'CDCLIENTE'       => $cdCliente,
            'IS_CLIENT'       => $isClient
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findAppliedDeployInfo($deployId)
    {
        $query = <<<SQL
            SELECT DP.ID, DP.PRODUCT_VERSION_ID, DP.PRODUCT_ID
              FROM (SELECT D.ID
                      FROM DEPLOY D
                     WHERE D.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
                     START WITH D.ID = :DEPLOY_ID
                   CONNECT BY PRIOR D.ID = D.PARENT) DL
              JOIN DEPLOY DP ON DP.ID = DL.ID
SQL;
        $params = array(
            'DEPLOY_ID' => $deployId,
            'IS_CUSTOMER_DEPLOY' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::IS_CUSTOMER_DEPLOY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getProductModules($productId, $customerCode)
    {
        $query = <<<SQL
        SELECT DISTINCT *
              FROM (SELECT PR.NAME, DP.PRODUCT_ID
                      FROM (SELECT D.ID
                              FROM DEPLOY D
                             WHERE D.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
                             START WITH D.PRODUCT_ID = :PRODUCT
                            CONNECT BY PRIOR D.ID = D.PARENT) DT
                     INNER JOIN DEPLOY DP ON DP.ID = DT.ID
                      LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DP.PRODUCT_VERSION_ID
                      LEFT JOIN PRODUCT_VERSION PP ON PP.ID = DP.PREVIOUS_PRODUCT_VERSION_ID
                      LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
                      LEFT JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
                      LEFT JOIN SERVER SR ON PS.SERVER_ID = SR.ID
                      LEFT JOIN PRODUCT PR ON DP.PRODUCT_ID = PR.ID
                      INNER JOIN RESOURCE_T RT ON RT.ID = DP.RESOURCE_T_ID
                                             AND RT.EXTERNAL_CODE = :EXTERNAL_CODE

                      LEFT JOIN ZH_USER ZC ON ZC.ID = DP.CREATED_BY
                      LEFT JOIN RESOURCE_T RTC ON RTC.ID = ZC.EXTERNAL_RES_ID
                      LEFT JOIN OPERADOR OPC ON OPC.CDOPERADOR = RTC.EXTERNAL_CODE

                      LEFT JOIN ZH_USER ZU ON ZU.ID = DP.UPDATED_BY
                      LEFT JOIN RESOURCE_T RTU ON RTU.ID = ZU.EXTERNAL_RES_ID
                      LEFT JOIN OPERADOR OPU ON OPU.CDOPERADOR = RTU.EXTERNAL_CODE

                     WHERE DP.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
                       AND DP.IS_DEPLOY_STEP = 'N'
                  ORDER BY DP.DEPLOY_DATE DESC)
             WHERE 1 = 1
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'EXTERNAL_CODE' => $customerCode,
            'IS_CUSTOMER_DEPLOY' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::IS_CUSTOMER_DEPLOY
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findServerInfoReturnBkpDeployStep($deployStepId, $productServerId)
    {
        $query = <<<SQL
            SELECT DEP.ID, DEP.NAME,
                   DEP.PRODUCT_ID, PRO.NAME AS PRODUCT_NAME,
                   SRV.WSEMAIL, SRV.NAME AS SERVER_NAME,
                   DSS.PRODUCT_SERVER_ID, PRS.PATH AS BASE_PATH,
                   PRV.VERSION || '.' || PRV.REVISION  || '.' || PRV.BUILD AS VERSION
              FROM DEPLOY_STEP_SERVER DSS
              JOIN DEPLOY DEP          ON DEP.ID = DSS.DEPLOY_ID
              JOIN PRODUCT PRO         ON PRO.ID = DEP.PRODUCT_ID
              JOIN PRODUCT_VERSION PRV ON PRV.ID = DEP.PRODUCT_VERSION_ID
              JOIN PRODUCT_SERVER PRS  ON PRS.ID = DSS.PRODUCT_SERVER_ID
              JOIN SERVER SRV          ON SRV.ID = PRS.SERVER_ID
             WHERE DSS.DEPLOY_STEP_ID    = :DEPLOY_STEP_ID
               AND DSS.PRODUCT_SERVER_ID = :PRODUCT_SERVER_ID
SQL;
        $params = array(
            'DEPLOY_STEP_ID'    => $deployStepId,
            'PRODUCT_SERVER_ID' => $productServerId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findOpenedCustomerDeploy($productId, $customerCode, $productServerId)
    {
        $query = <<<SQL
            SELECT DP.ID, DP.NAME
              FROM DEPLOY DP
              LEFT JOIN DEPLOY_SERVER DS ON DS.DEPLOY_ID = DP.ID
              LEFT JOIN PRODUCT_SERVER PS ON PS.ID = DS.PRODUCT_SERVER_ID
             INNER JOIN RESOURCE_T RT ON RT.ID = DP.RESOURCE_T_ID
                                     AND RT.EXTERNAL_CODE = :EXTERNAL_CODE
             WHERE DP.PRODUCT_ID = :PRODUCT
               AND DP.IS_ACTIVE = :IS_ACTIVE
               AND DP.IS_CUSTOMER_DEPLOY = :IS_CUSTOMER_DEPLOY
               AND DS.PRODUCT_SERVER_ID = :PRODUCT_SERVER_ID
               AND DP.STATUS IN (:OPENED_STATUS)
SQL;
        $params = array(
            'PRODUCT' => $productId,
            'EXTERNAL_CODE' => $customerCode,
            'PRODUCT_SERVER_ID' => $productServerId,
            'IS_ACTIVE' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::IS_ACTIVE_YES,
            'IS_CUSTOMER_DEPLOY' => \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::IS_CUSTOMER_DEPLOY,
            'OPENED_STATUS' => array(\Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::PENDING_DEPLOY_STATUS,
                \Zeedhi\DevOps\Model\Configurationmanager\Entities\Deploy::ERROR_DEPLOY_STATUS)
        );

        $paramType = array(
            'OPENED_STATUS' => Connection::PARAM_STR_ARRAY
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramType);
    }

    public function findLastDeployProduct($productId, $productServerId)
    {
        $query = "SELECT *
                    FROM (SELECT DEP.ID AS DEPLOY_ID, DEP.NAME AS DEPLOY_NAME,
                                 PVE.VERSION || '.' || PVE.REVISION || '.' || PVE.BUILD AS DEPLOY_VERSION, DEP.DEPLOY_DATE
                            FROM DEPLOY DEP
                      INNER JOIN PRODUCT_VERSION PVE ON PVE.ID = DEP.PRODUCT_VERSION_ID
                      INNER JOIN DEPLOY_SERVER DES ON DES.DEPLOY_ID = DEP.ID
                                                  AND DES.PRODUCT_SERVER_ID = :PRODUCT_SERVER_ID
                           WHERE DEP.PRODUCT_ID = :PRODUCT_ID
                             AND DEP.IS_ACTIVE  = 'Y'
                             AND DEP.STATUS     = 'D'
                        ORDER BY DEP.DEPLOY_DATE DESC)
                   WHERE ROWNUM = 1 ";

        $params = array(
            'PRODUCT_ID' => $productId,
            ':PRODUCT_SERVER_ID' => $productServerId
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function validateTicket($ticket)
    {
        $query = <<<SQL
        SELECT COUNT(*) AS COUNT FROM CONTATO WHERE NRSEQCONTATO = :ticket
SQL;

        $params = array(
            'ticket' => $ticket
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getKnowErrors($type)
    {
        $query = <<<SQL
        SELECT DESCRIPTION, RESOLUTION, REASON
          FROM ERROR_REPOSITORY
         WHERE IS_ACTIVE = 'Y'
           AND TYPE = :TYPE
      ORDER BY DESCRIPTION
SQL;

        $params = array(
            'TYPE' => $type
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findDeployUpdates($filterQuery, $cdcliente, $deployId, $isClient)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT DECODE(I.ID, NULL, T.ID, T.ID) AS ID, DECODE(I.IS_ISSUE, NULL, T.IS_ISSUE, T.IS_ISSUE) AS IS_ISSUE,
                           PV.ID AS PRODUCT_VERSION_ID, DECODE(PV.VERSION, NULL, '-', PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD) AS VERSION,
                           PV.VERSION AS PV_VERSION, PV.REVISION AS PV_REVISION, PV.BUILD AS PV_BUILD,
                           DECODE(I.NAME, NULL, T.NAME, I.NAME)  AS NAME,
                           DECODE(I.RESPONSE, NULL, T.RESPONSE, I.RESPONSE) AS RESPONSE,
                           D.PRODUCT_NAME || ' ' || D.INTERVAL_VERSION AS PRODUCT_NAME, D.PRODUCT_ID,
                           D.IS_PRODUCT,
                           CASE WHEN DECODE(I.ID, NULL, RT.EXTERNAL_CODE, RI.EXTERNAL_CODE) = :CDCLIENTE
                                THEN 'S'
                                ELSE 'N'
                                 END AS IS_CLIENT
                      FROM (SELECT DP.ID, DP.NAME, DP.PRODUCT_ID,
                                   P.VERSION, P.REVISION, P.BUILD, P.ID AS PRODUCT_VERSION_ID,
                                   PR.NAME AS PRODUCT_NAME,
                                   '(' ||
                                   CASE
                                         WHEN PV.VERSION IS NULL THEN ''
                                         WHEN TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) > TO_NUMBER(LPAD(P.VERSION, 5, '0') || LPAD(P.REVISION, 5, '0') || LPAD(P.BUILD, 5, '0')) THEN ''
                                         ELSE PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD
                                    END ||
                                   CASE
                                        WHEN PV.VERSION IS NULL THEN ''
                                        WHEN P.VERSION IS NULL THEN ''
                                        WHEN TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) > TO_NUMBER(LPAD(P.VERSION, 5, '0') || LPAD(P.REVISION, 5, '0') || LPAD(P.BUILD, 5, '0')) THEN ''
                                        ELSE ' a '
                                    END ||
                                   DECODE(P.VERSION, NULL, '', P.VERSION || '.' || P.REVISION  || '.' || P.BUILD) || ')' AS INTERVAL_VERSION,
                                   PV.VERSION AS PREVIOUS_VERSION, PV.REVISION PREVIOUS_REVISION, PV.BUILD AS PREVIOUS_BUILD, PV.ID AS PREVIOUS_PRODUCT_VERSION_ID,
                                   'S' AS IS_PRODUCT
                              FROM DEPLOY DP
                              JOIN PRODUCT_VERSION P ON P.ID = DP.PRODUCT_VERSION_ID
                         LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DP.PREVIOUS_PRODUCT_VERSION_ID
                              JOIN PRODUCT PR ON DP.PRODUCT_ID = PR.ID
                             WHERE DP.ID = :DEPLOY_ID

                             UNION

                            SELECT DE.ID, DE.NAME, DE.PRODUCT_ID,
                                   P.VERSION, P.REVISION, P.BUILD, P.ID AS PRODUCT_VERSION_ID,
                                   PR.NAME AS PRODUCT_NAME,
                                   '(' ||
                                   CASE
                                         WHEN PV.VERSION IS NULL THEN ''
                                         WHEN TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) > TO_NUMBER(LPAD(P.VERSION, 5, '0') || LPAD(P.REVISION, 5, '0') || LPAD(P.BUILD, 5, '0')) THEN ''
                                         ELSE PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD
                                    END ||
                                   CASE
                                        WHEN PV.VERSION IS NULL THEN ''
                                        WHEN P.VERSION IS NULL THEN ''
                                        WHEN TO_NUMBER(LPAD(PV.VERSION, 5, '0') || LPAD(PV.REVISION, 5, '0') || LPAD(PV.BUILD, 5, '0')) > TO_NUMBER(LPAD(P.VERSION, 5, '0') || LPAD(P.REVISION, 5, '0') || LPAD(P.BUILD, 5, '0')) THEN ''
                                        ELSE ' a '
                                    END ||
                                   DECODE(P.VERSION, NULL, '-', P.VERSION || '.' || P.REVISION  || '.' || P.BUILD) || ')' AS INTERVAL_VERSION,
                                   PV.VERSION AS PREVIOUS_VERSION, PV.REVISION PREVIOUS_REVISION, PV.BUILD AS PREVIOUS_BUILD, PV.ID AS PREVIOUS_PRODUCT_VERSION_ID,
                                   'N' AS IS_PRODUCT
                              FROM DEPLOY DE
                              JOIN PRODUCT_VERSION P ON P.ID = DE.PRODUCT_VERSION_ID
                         LEFT JOIN PRODUCT_VERSION PV ON PV.ID = DE.PREVIOUS_PRODUCT_VERSION_ID
                              JOIN PRODUCT PR ON DE.PRODUCT_ID = PR.ID
                             WHERE DE.PARENT = :DEPLOY_ID) D
                   JOIN PRODUCT_VERSION PV ON (PV.ID = D.PRODUCT_VERSION_ID AND (D.PRODUCT_VERSION_ID <> D.PREVIOUS_PRODUCT_VERSION_ID OR D.PREVIOUS_PRODUCT_VERSION_ID IS NULL))
                                            OR PV.ID IN (SELECT ID
                                                           FROM PRODUCT_VERSION PRV
                                                          WHERE D.PREVIOUS_PRODUCT_VERSION_ID IS NOT NULL
                                                            AND PRV.PRODUCT_ID = D.PRODUCT_ID
                                                            AND PRV.ID <> D.PREVIOUS_PRODUCT_VERSION_ID
                                                            AND TO_NUMBER(LPAD(PRV.VERSION, 5, '0') || LPAD(PRV.REVISION, 5, '0') || LPAD(PRV.BUILD, 5, '0')) BETWEEN
                                                                TO_NUMBER(LPAD(D.PREVIOUS_VERSION, 5, '0') || LPAD(D.PREVIOUS_REVISION, 5, '0') || LPAD(D.PREVIOUS_BUILD, 5, '0')) AND
                                                                TO_NUMBER(LPAD(D.VERSION, 5, '0') || LPAD(D.REVISION, 5, '0') || LPAD(D.BUILD, 5, '0')))
                  JOIN RELEASE R ON R.PRODUCT_ID = D.PRODUCT_ID
                  JOIN SUBRELEASE S ON S.RELEASE_ID = R.ID
                                   AND S.VERSION  = PV.VERSION
                                   AND S.REVISION = PV.REVISION
                                   AND S.BUILD    = PV.BUILD
                  JOIN BRANCH B ON B.SUBRELEASE_ID = S.ID
                  JOIN TASK_BRANCHES TB ON TB.NAME = B.NAME
                  JOIN TASK T ON TB.TASK = T.ID
                  JOIN CLASSIFICATION C ON C.ID = T.CLASSIFICATION AND (C.USED_IN_NEWS = 'Y' OR (C.CONTROL IS NULL AND  :IS_CLIENT = 'S'))

             LEFT JOIN TASK_RELATIONSHIP TR ON TR.TASK_RELATIONSHIP = T.ID AND TR.TYPE = 'I'
             LEFT JOIN TASK I ON I.ID = TR.TASK
             LEFT JOIN ALLOCATION A ON A.TASK = I.ID AND A.TYPE = 'S'
             LEFT JOIN RESOURCE_T RI ON RI.ID = A.RESOURCE_T
             LEFT JOIN RESOURCE_PROJECT RP ON RP.PROJECT = T.PROJECT AND RP.TYPE = 'S'
             LEFT JOIN RESOURCE_T RT ON RT.ID = RP.RESOURCE_C)
             WHERE 1 = 1
               AND IS_CLIENT = :IS_CLIENT
             $filterQuery
          ORDER BY PV_VERSION, PV_REVISION, PV_BUILD
SQL;
        $params = array(
            'DEPLOY_ID' => $deployId,
            'CDCLIENTE' => $cdcliente,
            'IS_CLIENT' => $isClient
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findProductsByDeploy($deployId, $filterQuery)
    {
        $query = <<<SQL
            SELECT *
              FROM (SELECT OP.ID,
                           DP.NAME, DP.PRODUCT_ID
                      FROM DEPLOY DP
                      JOIN OM_PRODUTO OP ON OP.NREXTERNAL_PRODUCT_ID = DP.PRODUCT_ID
                     WHERE DP.ID = :DEPLOY_ID

                     UNION

                     SELECT OP.ID,
                            DE.NAME, DE.PRODUCT_ID
                       FROM DEPLOY DE
                       JOIN OM_PRODUTO OP ON OP.NREXTERNAL_PRODUCT_ID = DE.PRODUCT_ID
                      WHERE DE.PARENT = :DEPLOY_ID)
            WHERE 1 = 1
             $filterQuery
SQL;
        $params = array(
            'DEPLOY_ID' => $deployId
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function findUpdatesBeforeDeploy($productId, $version, $revision, $build, $actualVersion, $actualRevision,
                                            $actualBuild, $productVersionId, $actualProductVersionId, $cdcliente, $isClient)
    {
        $productVersionId = $productVersionId ? $productVersionId : 'NULL';
        $actualProductVersionId = $actualProductVersionId ? $actualProductVersionId : 'NULL';
        $version = ($version || $version == 0) && !is_null($version) ? $version : 'NULL';
        $revision = ($revision || $revision == 0) && !is_null($revision) ? $revision : 'NULL';
        $build = ($build || $build == 0) && !is_null($build) ? $build : 'NULL';
        $actualVersion = ($actualVersion || $actualVersion == 0) && !is_null($actualVersion) ? $actualVersion : 'NULL';
        $actualRevision = ($actualRevision || $actualRevision == 0) && !is_null($actualRevision) ? $actualRevision : 'NULL';
        $actualBuild = ($actualBuild || $actualBuild == 0) && !is_null($actualBuild) ? $actualBuild : 'NULL';

        $query = <<<SQL
        SELECT *
           FROM (SELECT DECODE(I.ID, NULL, T.ID, T.ID) AS ID,
                        PV.ID AS PRODUCT_VERSION_ID, DECODE(PV.VERSION, NULL, '-', PV.VERSION || '.' || PV.REVISION  || '.' || PV.BUILD) AS VERSION,
                        PV.VERSION AS PV_VERSION, PV.REVISION AS PV_REVISION, PV.BUILD AS PV_BUILD,
                        DECODE(I.NAME, NULL, T.NAME, I.NAME)  AS NAME,
                        DECODE(I.RESPONSE, NULL, T.RESPONSE, I.RESPONSE) AS RESPONSE,
                        P.NAME || ' (' || CASE
                                                WHEN $actualProductVersionId IS NULL THEN ''
                                                WHEN TO_NUMBER(LPAD($actualVersion, 5, '0') || LPAD($actualRevision, 5, '0') || LPAD($actualBuild, 5, '0')) > TO_NUMBER(LPAD($version, 5, '0') || LPAD($revision, 5, '0') || LPAD($build, 5, '0')) THEN ''
                                                ELSE $actualVersion || '.' || $actualRevision  || '.' || $actualBuild
                                           END ||
                                          CASE
                                                WHEN $actualProductVersionId IS NULL THEN ''
                                                WHEN $productVersionId IS NULL THEN ''
                                                WHEN TO_NUMBER(LPAD($actualVersion, 5, '0') || LPAD($actualRevision, 5, '0') || LPAD($actualBuild, 5, '0')) > TO_NUMBER(LPAD($version, 5, '0') || LPAD($revision, 5, '0') || LPAD($build, 5, '0')) THEN ''
                                                ELSE ' a '
                                            END ||
                                          DECODE($productVersionId, NULL, '', $version || '.' || $revision  || '.' || $build) || ')' AS PRODUCT_NAME,
                        CASE WHEN DECODE(I.ID, NULL, RT.EXTERNAL_CODE, RI.EXTERNAL_CODE) = '$cdcliente'
                             THEN 'S'
                             ELSE 'N'
                              END AS IS_CLIENT
                   FROM PRODUCT P

                   JOIN PRODUCT_VERSION PV ON (PV.ID = $productVersionId AND ($productVersionId <> $actualProductVersionId OR $actualProductVersionId IS NULL))
                                            OR PV.ID IN (SELECT ID
                                                           FROM PRODUCT_VERSION PRV
                                                          WHERE $actualProductVersionId IS NOT NULL
                                                            AND PRV.PRODUCT_ID = $productId
                                                            AND PRV.ID <> $actualProductVersionId
                                                            AND TO_NUMBER(LPAD(PRV.VERSION, 5, '0') || LPAD(PRV.REVISION, 5, '0') || LPAD(PRV.BUILD, 5, '0')) BETWEEN
                                                                TO_NUMBER(LPAD($actualVersion, 5, '0') || LPAD($actualRevision, 5, '0') || LPAD($actualBuild, 5, '0')) AND
                                                                TO_NUMBER(LPAD($version, 5, '0') || LPAD($revision, 5, '0') || LPAD($build, 5, '0')))

                   JOIN RELEASE R ON R.PRODUCT_ID = P.ID
                   JOIN SUBRELEASE S ON S.RELEASE_ID = R.ID
                               AND S.VERSION  = PV.VERSION
                               AND S.REVISION = PV.REVISION
                               AND S.BUILD    = PV.BUILD
                   JOIN BRANCH B ON B.SUBRELEASE_ID = S.ID
                   JOIN TASK_BRANCHES TB ON TB.NAME = B.NAME
                   JOIN TASK T ON TB.TASK = T.ID
                   JOIN CLASSIFICATION C ON C.ID = T.CLASSIFICATION AND (C.USED_IN_NEWS = 'Y' OR (C.CONTROL IS NULL AND  '$isClient' = 'S'))
              LEFT JOIN TASK_RELATIONSHIP TR ON TR.TASK_RELATIONSHIP = T.ID AND TR.TYPE = 'I'
              LEFT JOIN TASK I ON I.ID = TR.TASK
              LEFT JOIN ALLOCATION A ON A.TASK = I.ID AND A.TYPE = 'S'
              LEFT JOIN RESOURCE_T RI ON RI.ID = A.RESOURCE_T
              LEFT JOIN RESOURCE_PROJECT RP ON RP.PROJECT = T.PROJECT AND RP.TYPE = 'S'
              LEFT JOIN RESOURCE_T RT ON RT.ID = RP.RESOURCE_C
                  WHERE P.ID IN ($productId))
        WHERE 1 = 1
          AND IS_CLIENT = '$isClient'

        UNION
SQL;
        return $query;
    }

    public function applyQueryFindProductsByDeploy($query)
    {
        return $this->getEntityManager()->getConnection()->fetchAll($query);
    }
}
